@@ -2,7 +2,7 @@
 
 #ifndef LISTH
 #define LISTH
-/*	Copyright (C) 1995,1996,1997, 2000, 2001 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997, 2000, 2001, 2006 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -54,6 +54,7 @@ extern SCM scm_list_3 (SCM e1, SCM e2, SCM e3);
 extern SCM scm_list_4 (SCM e1, SCM e2, SCM e3, SCM e4);
 extern SCM scm_list_5 (SCM e1, SCM e2, SCM e3, SCM e4, SCM e5);
 extern SCM scm_list_n (SCM elt, ...);
+extern SCM scm_list (SCM objs);
 extern SCM scm_list_head (SCM lst, SCM k);
 extern SCM scm_list (SCM objs);
 extern SCM scm_cons_star (SCM arg, SCM objs);
