@@ -1,9 +1,9 @@
 /* classes: h_files */
 
-#ifndef OBJECTSH
-#define OBJECTSH
+#ifndef SCM_OBJECTS_H
+#define SCM_OBJECTS_H
 
-/*	Copyright (C) 1996, 1999, 2000 Free Software Foundation, Inc.
+/* Copyright (C) 1996, 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -94,13 +94,13 @@
 #define SCM_SET_ENTITY_PROCEDURE(obj,v) \
         (SCM_STRUCT_DATA (obj) [scm_struct_i_procedure] = SCM_UNPACK (v))
 #define SCM_ENTITY_SETTER(obj) (SCM_PACK (SCM_STRUCT_DATA (obj)[scm_struct_i_setter]))
+#define SCM_SET_ENTITY_SETTER(obj, v) \
+        (SCM_STRUCT_DATA (obj) [scm_struct_i_setter] = SCM_UNPACK (v))
 
 #define SCM_SET_CLASS_DESTRUCTOR(c, d) SCM_SET_VTABLE_DESTRUCTOR (c, d)
 #define SCM_SET_CLASS_INSTANCE_SIZE(c, s) \
-(SCM_STRUCT_DATA (c)[scm_struct_i_size] \
- = SCM_PACK ((SCM_UNPACK (SCM_STRUCT_DATA (c)[scm_struct_i_size])\
-               & SCM_STRUCTF_MASK)\
-              | s))
+  (SCM_STRUCT_DATA (c)[scm_struct_i_size] \
+   = (SCM_STRUCT_DATA (c) [scm_struct_i_size] & SCM_STRUCTF_MASK) | s)
 
 /* {Operator classes}
  *
@@ -207,14 +207,15 @@ extern SCM scm_class_real, scm_class_complex, scm_class_integer;
 extern SCM scm_class_unknown;
 extern SCM *scm_port_class;
 extern SCM *scm_smob_class;
+extern int scm_classes_initialized;
 
 extern SCM scm_no_applicable_method;
 
-/* Plugin Goops functions. */
-extern SCM (*scm_make_extended_class) (char *type_name);
-extern void (*scm_make_port_classes) (int ptobnum, char *type_name);
-extern void (*scm_change_object_class) (SCM, SCM, SCM);
-extern SCM (*scm_memoize_method) (SCM x, SCM args);
+/* Goops functions. */
+extern SCM scm_make_extended_class (char *type_name);
+extern void scm_make_port_classes (long ptobnum, char *type_name);
+extern void scm_change_object_class (SCM, SCM, SCM);
+extern SCM scm_memoize_method (SCM x, SCM args);
 
 extern SCM scm_class_of (SCM obj);
 extern SCM scm_mcache_lookup_cmethod (SCM cache, SCM args);
@@ -228,7 +229,8 @@ extern SCM scm_apply_generic (SCM gf, SCM args);
 extern SCM scm_call_generic_3 (SCM gf, SCM a1, SCM a2, SCM a3);
 extern SCM scm_entity_p (SCM obj);
 extern SCM scm_operator_p (SCM obj);
-extern SCM scm_set_object_procedure_x (SCM obj, SCM procs);
+extern SCM scm_valid_object_procedure_p (SCM proc);
+extern SCM scm_set_object_procedure_x (SCM obj, SCM proc);
 #ifdef GUILE_DEBUG
 extern SCM scm_object_procedure (SCM obj);
 #endif
@@ -239,7 +241,7 @@ extern SCM scm_i_make_class_object (SCM metaclass, SCM layout_string,
 				    unsigned long flags);
 extern void scm_init_objects (void);
 
-#endif /* OBJECTSH */
+#endif /* SCM_OBJECTS_H */
 
 /*
   Local Variables:
