@@ -1,8 +1,8 @@
 /* classes: h_files */
 
-#ifndef PORTSH
-#define PORTSH
-/*	Copyright (C) 1995,1996,1997,1998,1999, 2000, 2002 Free Software Foundation, Inc.
+#ifndef SCM_PORTS_H
+#define SCM_PORTS_H
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -57,26 +57,26 @@
 #define SCM_INITIAL_PUTBACK_BUF_SIZE 4
 
 /* values for the rw_active flag.  */
-enum scm_port_rw_active {
+typedef enum scm_t_port_rw_active {
   SCM_PORT_NEITHER = 0,
   SCM_PORT_READ = 1,
   SCM_PORT_WRITE = 2
-};
+} scm_t_port_rw_active;
 
 /* C representation of a Scheme port.  */
 
 typedef struct 
 {
   SCM port;			/* Link back to the port object.  */
-  int entry;			/* Index in port table. */
+  long entry;			/* Index in port table. */
   int revealed;			/* 0 not revealed, > 1 revealed.
 				 * Revealed ports do not get GC'd.
 				 */
   /* data for the underlying port implementation as a raw C value. */
-  scm_bits_t stream;
+  scm_t_bits stream;
 
   SCM file_name;		/* debugging support.  */
-  int line_number;		/* debugging support.  */
+  long line_number;		/* debugging support.  */
   int column_number;		/* debugging support.  */
 
   /* port buffers.  the buffer(s) are set up for all ports.  
@@ -118,21 +118,22 @@ typedef struct
 				   flushed before switching between
 				   reading and writing, seeking, etc.  */
 
-  enum scm_port_rw_active rw_active; /* for random access ports,
-					indicates which of the buffers
-					is currently in use.  can be
-					SCM_PORT_WRITE, SCM_PORT_READ,
-					or SCM_PORT_NEITHER.  */
+  scm_t_port_rw_active rw_active; /* for random access ports,
+                                     indicates which of the buffers
+                                     is currently in use.  can be
+                                     SCM_PORT_WRITE, SCM_PORT_READ,
+                                     or SCM_PORT_NEITHER.  */
 
 
   /* a buffer for un-read chars and strings.  */
   unsigned char *putback_buf;
-  int putback_buf_size;        /* allocated size of putback_buf.  */
-} scm_port;
+  size_t putback_buf_size;        /* allocated size of putback_buf.  */
+} scm_t_port;
 
-extern scm_port **scm_port_table;
-extern int scm_port_table_size; /* Number of ports in scm_port_table.  */
+extern scm_t_port **scm_port_table;
+extern long scm_port_table_size; /* Number of ports in scm_port_table.  */
 
+#define SCM_READ_BUFFER_EMPTY_P(c_port) (c_port->read_pos >= c_port->read_end)
 
 
 
@@ -149,7 +150,7 @@ extern int scm_port_table_size; /* Number of ports in scm_port_table.  */
 #define SCM_BUF0	(8L<<16) /* Is it unbuffered? */
 #define SCM_BUFLINE     (64L<<16) /* Is it line-buffered? */
 
-#define SCM_PORTP(x) (SCM_NIMP(x) && (SCM_TYP7(x)==scm_tc7_port))
+#define SCM_PORTP(x) (SCM_NIMP (x) && (SCM_TYP7 (x) == scm_tc7_port))
 #define SCM_OPPORTP(x) (SCM_NIMP(x) && (((0x7f | SCM_OPN) & SCM_CELL_WORD_0(x))==(scm_tc7_port | SCM_OPN)))
 #define SCM_OPINPORTP(x) (SCM_NIMP(x) && (((0x7f | SCM_OPN | SCM_RDNG) & SCM_CELL_WORD_0(x))==(scm_tc7_port | SCM_OPN | SCM_RDNG)))
 #define SCM_OPOUTPORTP(x) (SCM_NIMP(x) && (((0x7f | SCM_OPN | SCM_WRTNG) & SCM_CELL_WORD_0(x))==(scm_tc7_port | SCM_OPN | SCM_WRTNG)))
@@ -161,29 +162,34 @@ extern int scm_port_table_size; /* Number of ports in scm_port_table.  */
    && (((0x7f | SCM_WRTNG) & SCM_CELL_WORD_0(x))==(scm_tc7_port | SCM_WRTNG)))
 #define SCM_OPENP(x) (SCM_NIMP(x) && (SCM_OPN & SCM_CELL_WORD_0 (x)))
 #define SCM_CLOSEDP(x) (!SCM_OPENP(x))
+#define SCM_CLR_PORT_OPEN_FLAG(p) \
+  SCM_SET_CELL_WORD_0 ((p), SCM_CELL_WORD_0 (p) & ~SCM_OPN)
 
-#define SCM_PTAB_ENTRY(x)         ((scm_port *) SCM_CELL_WORD_1 (x))
-#define SCM_SETPTAB_ENTRY(x,ent)  (SCM_SET_CELL_WORD_1 ((x), (scm_bits_t) (ent)))
+#define SCM_PTAB_ENTRY(x)         ((scm_t_port *) SCM_CELL_WORD_1 (x))
+#define SCM_SETPTAB_ENTRY(x,ent)  (SCM_SET_CELL_WORD_1 ((x), (scm_t_bits) (ent)))
 #define SCM_STREAM(x)             (SCM_PTAB_ENTRY(x)->stream)
-#define SCM_SETSTREAM(x,s)        (SCM_PTAB_ENTRY(x)->stream = (scm_bits_t) (s))
+#define SCM_SETSTREAM(x,s)        (SCM_PTAB_ENTRY(x)->stream = (scm_t_bits) (s))
 #define SCM_FILENAME(x)           (SCM_PTAB_ENTRY(x)->file_name)
+#define SCM_SET_FILENAME(x, n)    (SCM_PTAB_ENTRY(x)->file_name = (n))
 #define SCM_LINUM(x)              (SCM_PTAB_ENTRY(x)->line_number)
 #define SCM_COL(x)                (SCM_PTAB_ENTRY(x)->column_number)
 #define SCM_REVEALED(x)           (SCM_PTAB_ENTRY(x)->revealed)
 #define SCM_SETREVEALED(x,s)      (SCM_PTAB_ENTRY(x)->revealed = (s))
 
 #define SCM_INCLINE(port)  	{SCM_LINUM (port) += 1; SCM_COL (port) = 0;}
+#define SCM_ZEROCOL(port)  	{SCM_COL (port) = 0;}
 #define SCM_INCCOL(port)  	{SCM_COL (port) += 1;}
+#define SCM_DECCOL(port)  	{if (SCM_COL (port) > 0) SCM_COL (port) -= 1;}
 #define SCM_TABCOL(port)  	{SCM_COL (port) += 8 - SCM_COL (port) % 8;}
 
 
 
 /* port-type description.  */
-typedef struct scm_ptob_descriptor
+typedef struct scm_t_ptob_descriptor
 {
   char *name;
   SCM (*mark) (SCM);
-  scm_sizet (*free) (SCM);
+  size_t (*free) (SCM);
   int (*print) (SCM exp, SCM port, scm_print_state *pstate);
   SCM (*equalp) (SCM, SCM);
   int (*close) (SCM port);
@@ -198,7 +204,13 @@ typedef struct scm_ptob_descriptor
   off_t (*seek) (SCM port, off_t OFFSET, int WHENCE);
   void (*truncate) (SCM port, off_t length);
 
-} scm_ptob_descriptor;
+} scm_t_ptob_descriptor;
+
+#if (SCM_DEBUG_DEPRECATED == 0)
+# define scm_port scm_t_port
+# define scm_ptob_descriptor scm_t_ptob_descriptor
+# define scm_port_rw_active scm_t_port_rw_active
+#endif
 
 #define SCM_TC2PTOBNUM(x) (0x0ff & ((x) >> 8))
 #define SCM_PTOBNUM(x) (SCM_TC2PTOBNUM (SCM_CELL_TYPE (x)))
@@ -207,19 +219,20 @@ typedef struct scm_ptob_descriptor
 
 
 
-extern scm_ptob_descriptor *scm_ptobs;
-extern int scm_numptob;
-extern int scm_port_table_room;
+extern scm_t_ptob_descriptor *scm_ptobs;
+extern long scm_numptob;
+extern long scm_port_table_room;
 
 
 
 extern SCM scm_markstream (SCM ptr);
-extern long scm_make_port_type (char *name,
-				int (*fill_input) (SCM port),
-				void (*write) (SCM port, const void *data,
-					       size_t size));
+extern scm_t_bits scm_make_port_type (char *name,
+				      int (*fill_input) (SCM port),
+				      void (*write) (SCM port, 
+						     const void *data,
+						     size_t size));
 extern void scm_set_port_mark (long tc, SCM (*mark) (SCM));
-extern void scm_set_port_free (long tc, scm_sizet (*free) (SCM));
+extern void scm_set_port_free (long tc, size_t (*free) (SCM));
 extern void scm_set_port_print (long tc,
 				int (*print) (SCM exp,
 					      SCM port,
@@ -241,6 +254,7 @@ extern void scm_set_port_truncate (long tc,
 						     off_t length));
 extern void scm_set_port_input_waiting (long tc, int (*input_waiting) (SCM));
 extern SCM scm_char_ready_p (SCM port);
+size_t scm_take_from_input_buffers (SCM port, char *dest, size_t read_len);
 extern SCM scm_drain_input (SCM port);
 extern SCM scm_current_input_port (void);
 extern SCM scm_current_output_port (void);
@@ -249,12 +263,12 @@ extern SCM scm_current_load_port (void);
 extern SCM scm_set_current_input_port (SCM port);
 extern SCM scm_set_current_output_port (SCM port);
 extern SCM scm_set_current_error_port (SCM port);
-extern scm_port * scm_add_to_port_table (SCM port);
+extern scm_t_port * scm_add_to_port_table (SCM port);
 extern void scm_remove_from_port_table (SCM port);
 extern void scm_grow_port_cbuf (SCM port, size_t requested);
 extern SCM scm_pt_size (void);
 extern SCM scm_pt_member (SCM member);
-extern void scm_port_non_buffer (scm_port *pt);
+extern void scm_port_non_buffer (scm_t_port *pt);
 extern int scm_revealed_count (SCM port);
 extern SCM scm_port_revealed (SCM port);
 extern SCM scm_set_port_revealed_x (SCM port, SCM rcount);
@@ -263,9 +277,10 @@ extern SCM scm_port_mode (SCM port);
 extern SCM scm_close_input_port (SCM port);
 extern SCM scm_close_output_port (SCM port);
 extern SCM scm_close_port (SCM port);
-extern SCM scm_close_all_ports_except (SCM ports);
+extern SCM scm_port_for_each (SCM proc);
 extern SCM scm_input_port_p (SCM x);
 extern SCM scm_output_port_p (SCM x);
+extern SCM scm_port_p (SCM x);
 extern SCM scm_port_closed_p (SCM port);
 extern SCM scm_eof_object_p (SCM x);
 extern SCM scm_force_output (SCM port);
@@ -273,7 +288,9 @@ extern SCM scm_flush_all_ports (void);
 extern SCM scm_read_char (SCM port);
 extern void scm_putc (char c, SCM port);
 extern void scm_puts (const char *str_data, SCM port);
-extern void scm_lfwrite (const char *ptr, scm_sizet size, SCM port);
+extern size_t scm_c_read (SCM port, void *buffer, size_t size);
+extern void scm_c_write (SCM port, const void *buffer, size_t size);
+extern void scm_lfwrite (const char *ptr, size_t size, SCM port);
 extern void scm_flush (SCM port);
 extern void scm_end_input (SCM port);
 extern int scm_fill_input (SCM port);
@@ -307,13 +324,11 @@ extern SCM scm_pt_member (SCM member);
 
 #if (SCM_DEBUG_DEPRECATED == 0)
 
-/* #define SCM_CRDY	(32L<<16)  obsolete, for pushed back characters  */
-#define SCM_INPORTP(x) SCM_INPUT_PORT_P (x)
-#define SCM_OUTPORTP(x) SCM_OUTPUT_PORT_P (x)
+extern SCM scm_close_all_ports_except (SCM ports);
 
 #endif  /* SCM_DEBUG_DEPRECATED == 0 */
 
-#endif  /* PORTSH */
+#endif  /* SCM_PORTS_H */
 
 /*
   Local Variables:
