@@ -1,4 +1,4 @@
-/* Copyright (C) 1994, 1995, 1996, 1997, 1998, 2000, 2001, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1994, 1995, 1996, 1997, 1998, 2000, 2001, 2002, 2004, 2005 Free Software Foundation, Inc.
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -11,8 +11,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -141,7 +141,7 @@ char *
 scm_find_executable (const char *name)
 {
   char tbuf[MAXPATHLEN];
-  int i = 0;
+  int i = 0, c;
   FILE *f;
 
   /* fprintf(stderr, "s_f_e checking access %s ->%d\n", name, access(name, X_OK)); fflush(stderr); */
@@ -153,16 +153,19 @@ scm_find_executable (const char *name)
   if ((fgetc (f) == '#') && (fgetc (f) == '!'))
     {
       while (1)
-	switch (tbuf[i++] = fgetc (f))
+	switch (c = fgetc (f))
 	  {
 	  case /*WHITE_SPACES */ ' ':
 	  case '\t':
 	  case '\r':
 	  case '\f':
 	  case EOF:
-	    tbuf[--i] = 0;
+	    tbuf[i] = 0;
 	    fclose (f);
 	    return scm_cat_path (0L, tbuf, 0L);
+	  default:
+	    tbuf[i++] = c;
+	    break;
 	  }
     }
   fclose (f);
@@ -366,10 +369,12 @@ char *scm_usage_name = 0;
 void
 scm_shell_usage (int fatal, char *message)
 {
-  if (message)
-    fprintf (stderr, "%s\n", message);
+  FILE  *fp = (fatal ? stderr : stdout);
 
-  fprintf (stderr, 
+  if (message)
+    fprintf (fp, "%s\n", message);
+
+  fprintf (fp, 
            "Usage: %s OPTION ...\n"
            "Evaluate Scheme code, interactively or from a script.\n"
            "\n"
@@ -390,7 +395,9 @@ scm_shell_usage (int fatal, char *message)
 	   "                 which is a list of numbers like \"2,13,14\"\n"
            "  -h, --help     display this help and exit\n"
            "  -v, --version  display version information and exit\n"
-	   "  \\              read arguments from following script lines\n",
+	   "  \\              read arguments from following script lines\n"
+           "\n"
+	   "Report bugs to bug-guile@gnu.org\n",
            scm_usage_name);
 
   if (fatal)
@@ -584,7 +591,7 @@ scm_compile_shell_switches (int argc, char **argv)
 	{
 	  /* Print version number.  */
 	  printf ("Guile %s\n"
-		  "Copyright (c) 1995, 1996, 1997, 2000, 2001, 2002 Free Software Foundation\n"
+		  "Copyright (c) 1995, 1996, 1997, 2000, 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation\n"
 		  "Guile may be distributed under the terms of the GNU General Public Licence;\n"
 		  "certain other uses are permitted as well.  For details, see the file\n"
 		  "`COPYING', which is included in the Guile distribution.\n"
