@@ -2,7 +2,7 @@
 
 #ifndef SIMPOSH
 #define SIMPOSH
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997,1998, 2000 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -47,9 +47,15 @@
 
 
 
-extern SCM scm_system SCM_P ((SCM cmd));
-extern SCM scm_getenv SCM_P ((SCM nam));
-extern SCM scm_software_type SCM_P ((void));
-extern void scm_init_simpos SCM_P ((void));
+extern SCM scm_system (SCM cmd);
+extern SCM scm_getenv (SCM nam);
+extern SCM scm_primitive_exit (SCM status);
+extern void scm_init_simpos (void);
 
 #endif  /* SIMPOSH */
+
+/*
+  Local Variables:
+  c-file-style: "gnu"
+  End:
+*/
