@@ -2,7 +2,7 @@
 
 #ifndef VPORTSH
 #define VPORTSH
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996, 2000 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -48,7 +48,13 @@
 
 
 
-extern SCM scm_make_soft_port SCM_P ((SCM pv, SCM modes));
-extern void scm_init_vports SCM_P ((void));
+extern SCM scm_make_soft_port (SCM pv, SCM modes);
+extern void scm_init_vports (void);
 
 #endif  /* VPORTSH */
+
+/*
+  Local Variables:
+  c-file-style: "gnu"
+  End:
+*/
