@@ -1,6 +1,6 @@
 ;;; installed-scm-file
 
-;;;; 	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+;;;; 	Copyright (C) 1998, 1999, 2000, 2001, 2003 Free Software Foundation, Inc.
 ;;;; 
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
 ;;;; 
 ;;;; You should have received a copy of the GNU General Public License
 ;;;; along with this software; see the file COPYING.  If not, write to
-;;;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
-;;;; Boston, MA 02111-1307 USA
+;;;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;;;; Boston, MA 02110-1301 USA
 ;;;;
 ;;;; As a special exception, the Free Software Foundation gives permission
 ;;;; for additional uses of the text contained in its release of GUILE.
@@ -508,7 +508,7 @@
 (define method
   (letrec ((specializers
 	    (lambda (ls)
-	      (cond ((null? ls) '('()))
+	      (cond ((null? ls) (list (list 'quote '())))
 		    ((pair? ls) (cons (if (pair? (car ls))
 					  (cadar ls)
 					  '<top>)
@@ -658,6 +658,12 @@
 ;;; Methods to compare objects
 ;;;
 
+(define-method (eqv? x y)   #f)
+(define-method (equal? x y) (eqv? x y))
+
+;;; These following two methods are for backward compatibility only.
+;;; They are not called by the Guile interpreter.
+;;;
 (define-method (object-eqv? x y)    #f)
 (define-method (object-equal? x y)  (eqv? x y))
 
@@ -909,7 +915,13 @@
 (define-method (remove-class-accessors! (c <class>))
   (for-each (lambda (m)
 	      (if (is-a? m <accessor-method>)
-		  (remove-method-in-classes! m)))
+		  (let ((gf (slot-ref m 'generic-function)))
+		    ;; remove the method from its GF
+		    (slot-set! gf 'methods
+			       (delq1! m (slot-ref gf 'methods)))
+		    (%invalidate-method-cache! gf)
+		    ;; remove the method from its specializers
+		    (remove-method-in-classes! m))))
 	    (class-direct-methods c)))
 
 ;;;
@@ -924,7 +936,7 @@
     ;; So we can work as if we had only proper lists.
     (if (pair? l)       	  
 	(begin
-	  (if (eqv? (car l) old)  
+	  (if (eq? (car l) old)  
 	      (set-car! l new))
 	  (loop (cdr l))))))
 
@@ -975,10 +987,7 @@
     (make <accessor-method>
           #:specializers (list class)
 	  #:procedure (cond ((pair? g-n-s)
-			     (if init-thunk
-				 (car g-n-s)
-				 (make-generic-bound-check-getter (car g-n-s))
-				 ))
+			     (make-generic-bound-check-getter (car g-n-s)))
 			    (init-thunk
 			     (standard-get g-n-s))
 			    (else
@@ -1033,37 +1042,64 @@
 (define standard-set (standard-accessor-method make-set standard-set-methods))
 
 ;;; compute-getters-n-setters
-;;; 
+;;;
+(define (make-thunk thunk)
+  (lambda () (thunk)))
+
 (define (compute-getters-n-setters class slots env)
 
-  (define (compute-slot-init-function s)
-    (or (slot-definition-init-thunk s)
+  (define (compute-slot-init-function name s)
+    (or (let ((thunk (slot-definition-init-thunk s)))
+	  (and thunk
+	       (cond ((not (thunk? thunk))
+		      (goops-error "Bad init-thunk for slot `~S' in ~S: ~S"
+				   name class thunk))
+		     ((closure? thunk) thunk)
+		     (else (make-thunk thunk)))))
 	(let ((init (slot-definition-init-value s)))
 	  (and (not (unbound? init))
 	       (lambda () init)))))
 
   (define (verify-accessors slot l)
-    (if (pair? l)
-	(let ((get (car l)) 
-	      (set (cadr l)))
-	  (if (not (and (closure? get)
-			(= (car (procedure-property get 'arity)) 1)))
-	      (goops-error "Bad getter closure for slot `~S' in ~S: ~S"
-			   slot class get))
-	  (if (not (and (closure? set)
-			(= (car (procedure-property set 'arity)) 2)))
-	    (goops-error "Bad setter closure for slot `~S' in ~S: ~S"
-			 slot class set)))))
+    (cond ((integer? l))
+	  ((not (and (list? l) (= (length l) 2)))
+	   (goops-error "Bad getter and setter for slot `~S' in ~S: ~S"
+			slot class l))
+	  (else
+	   (let ((get (car l)) 
+		 (set (cadr l)))
+	     (if (not (and (closure? get)
+			   (= (car (procedure-property get 'arity)) 1)))
+		 (goops-error "Bad getter closure for slot `~S' in ~S: ~S"
+			      slot class get))
+	     (if (not (and (closure? set)
+			   (= (car (procedure-property set 'arity)) 2)))
+		 (goops-error "Bad setter closure for slot `~S' in ~S: ~S"
+			      slot class set))))))
 
   (map (lambda (s)
-	 (let* ((g-n-s (compute-get-n-set class s))
+	 ;; The strange treatment of nfields is due to backward compatibility.
+	 (let* ((index (slot-ref class 'nfields))
+		(g-n-s (compute-get-n-set class s))
+		(size (- (slot-ref class 'nfields) index))
 		(name  (slot-definition-name s)))
-	   ; For each slot we have '(name init-function getter setter)
-	   ; If slot, we have the simplest form '(name init-function . index)
+	   ;; NOTE: The following is interdependent with C macros
+	   ;; defined above goops.c:scm_sys_prep_layout_x.
+	   ;;
+	   ;; For simple instance slots, we have the simplest form
+	   ;; '(name init-function . index)
+	   ;; For other slots we have
+	   ;; '(name init-function getter setter . alloc)
+	   ;; where alloc is:
+	   ;;   '(index size) for instance allocated slots
+	   ;;   '() for other slots
 	   (verify-accessors name g-n-s)
 	   (cons name
-		 (cons (compute-slot-init-function s)
-		       g-n-s))))
+		 (cons (compute-slot-init-function name s)
+		       (if (or (integer? g-n-s)
+			       (zero? size))
+			   g-n-s
+			   (append g-n-s (list index size)))))))
        slots))
 
 ;;; compute-cpl
